@@ -15,6 +15,7 @@ import pandas as pd
 
 # since our downloads are multi-threaded, we use a lock to avoid race conditions
 output_lock = threading.Lock()
+exit_code_lock = threading.Lock()
 
 
 def get_host(url):
@@ -64,11 +65,14 @@ def get_object_id_from_drs(url):
         raise Exception(f"Could not get drs object id from url {url}")
 
 
-def download_file(
-    url,
-    output_dir,
-    out: Optional[list] = None,
-):
+def exit_download(url, code, exit_codes):
+    if exit_codes is not None:
+        exit_code_lock.acquire()
+        exit_codes[url] = code
+        exit_code_lock.release()
+
+
+def download_file(url, output_dir, out: Optional[list] = None, exit_codes=None):
 
     http = urllib3.PoolManager()
     chunk_size = 1024
@@ -86,18 +90,18 @@ def download_file(
         except HTTPError as e:
             if e.response.status_code == 404:
                 click.secho(f"DRS object with id {object_id} does not exist", fg="red")
-                sys.exit(1)
             elif e.response.status_code == 403:
                 click.secho("Access Denied", fg="red")
-                sys.exit(1)
             else:
                 click.secho(
                     "There was an error getting object info from the DRS Client",
                     fg="red",
                 )
-                sys.exit(1)
+                exit_download(url, 1, exit_codes)
+                return
         except Exception as e:
-            sys.exit(1)
+            exit_download(url, 1, exit_codes)
+            return
 
         if "access_methods" in object_info.keys():
             for access_method in object_info["access_methods"][0]:
@@ -110,7 +114,7 @@ def download_file(
                             )
                         except HTTPError as e:
                             click.echo(e)
-                            sys.exit(1)
+                            exit_download(url, 1, exit_codes)
                         download_url = object_access["url"]
                         break
 
@@ -122,13 +126,15 @@ def download_file(
         else:
             return  # next page token, just return
     else:
-        click.secho(f'"{url}" is not a valid DRS url', fg="red")
+        click.secho(f"[{url}] is not a valid DRS url", fg="red")
+        exit_download(url, 1, exit_codes)
         return
 
     try:
         download_stream = http.request("GET", download_url, preload_content=False)
     except:
         click.secho("There was an error downloading " + download_url, fg="red")
+        exit_download(url, 1, exit_codes)
         return
 
     if out is not None:
@@ -140,14 +146,13 @@ def download_file(
     else:
         with open(f"{output_dir}/{download_file}", "wb+") as dest:
             stream_size = int(download_stream.headers["Content-Length"])
-            click.echo(
-                f"Downloading {url} into {output_dir}/{download_file} ", nl=False
-            )
+            click.echo(f"Downloading {url} into {output_dir}/{download_file}")
             file_stream = download_stream.stream(chunk_size)
-            with click.progressbar(length=stream_size) as download_progress:
+            with click.progressbar(length=stream_size, color=True) as download_progress:
                 for chunk in file_stream:
                     dest.write(chunk)
                     download_progress.update(chunk_size)
+    exit_download(url, 0, exit_codes)
 
 
 def download_files(
@@ -156,6 +161,7 @@ def download_files(
     out=None,
 ):
     download_threads = []
+    exit_codes = {}
 
     if not os.path.exists(output_dir):
         os.makedirs(output_dir)
@@ -166,6 +172,7 @@ def download_files(
                 url,
                 output_dir,
                 out=out,
+                exit_codes=exit_codes,
             ),
             name=url,
         )
@@ -175,3 +182,8 @@ def download_files(
 
     for thread in download_threads:
         thread.join()
+
+    # at least one download failed, raise an exception
+    failed_downloads = [url for url, code in exit_codes.items() if code != 0]
+    if len(failed_downloads) > 0:
+        raise Exception(f"One or more downloads failed: [{','.join(failed_downloads)}]")
