@@ -14,19 +14,17 @@ def handle_client_results(results, dataconnect_url):
     try:
         yield from results
     except SSLError:
-        click.secho(
-            f"There was an error retrieving the SSL certificate from {dataconnect_url}",
-            fg="red",
+        raise Exception(
+            f"There was an error retrieving the SSL certificate from {dataconnect_url}"
         )
-        sys.exit(1)
     except HTTPError as e:
         error_res = requests.get(e.response.url)
         error_json = json.loads(error_res.text)
-        error_msg = error_json["errors"][0]["title"]
-        click.secho(
-            f"There was an error querying from {dataconnect_url}: {error_msg}", fg="red"
-        )
-        sys.exit(1)
+        if "errors" in error_json:
+            error_msg = error_json["errors"][0]["title"]
+            raise Exception(error_msg)
+        else:
+            raise e
 
 
 def get_dataconnect_client(dataconnect_url):
